import {ClientData, Locations, Order, Pizza, PizzaDayState, PizzaOrder, PizzaSize, UdalostEnum} from "./types";
import {db} from "./database";
import {formatDate, getHumanDate, getIsWeekend} from "./utils";
import {callNotifikace} from "./notifikace";

/** Vrátí dnešní datum, případně fiktivní datum pro účely vývoje a testování. */
function getToday(): Date {
    if (process.env.MOCK_DATA) {
        return new Date('2023-05-31');
    }
    return new Date();
}

/** Vrátí "prázdná" (implicitní) data, pokud ještě nikdo nehlasoval. */
function getEmptyData(): ClientData {
    return { date: getHumanDate(getToday()), isWeekend: getIsWeekend(getToday()), choices: {} };
}

/**
 * Vrátí veškerá klientská data pro aktuální den.
 */
export function getData(): ClientData {
    const data = db.get(formatDate(getToday())) || getEmptyData();
    return data;
}

/**
 * Vytvoří pizza day pro aktuální den a vrátí data pro klienta.
 */
export function createPizzaDay(creator: string): ClientData {
    initIfNeeded();
    const today = formatDate(getToday());
    const clientData: ClientData = db.get(today);
    if (clientData.pizzaDay) {
        throw Error("Pizza day pro dnešní den již existuje");
    }
    const data: ClientData = { pizzaDay: { state: PizzaDayState.CREATED, creator, orders: [] }, ...clientData };
    db.set(today, data);
    callNotifikace({input:{udalost:UdalostEnum.ZAHAJENA_PIZZA,user:creator}})
    return data;
}

/**
 * Smaže pizza day pro aktuální den.
 */
export function deletePizzaDay(login: string) {
    const today = formatDate(getToday());
    const clientData: ClientData = db.get(today);
    if (!clientData.pizzaDay) {
        throw Error("Pizza day pro dnešní den neexistuje");
    }
    if (clientData.pizzaDay.creator !== login) {
        throw Error("Login uživatele se neshoduje se zakladatelem Pizza Day");
    }
    delete clientData.pizzaDay;
    db.set(today, clientData);
    return clientData;
}

/**
 * Přidá objednávku pizzy uživateli.
 * 
 * @param login login uživatele
 * @param pizza zvolená pizza
 * @param size zvolená velikost pizzy
 */
export function addPizzaOrder(login: string, pizza: Pizza, size: PizzaSize) {
    const today = formatDate(getToday());
    const clientData: ClientData = db.get(today);
    if (!clientData.pizzaDay) {
        throw Error("Pizza day pro dnešní den neexistuje");
    }
    if (clientData.pizzaDay.state !== PizzaDayState.CREATED) {
        throw Error("Pizza day není ve stavu " + PizzaDayState.CREATED);
    }
    let order: Order | undefined = clientData.pizzaDay.orders.find(o => o.customer === login);
    if (!order) {
        order = {
            customer: login,
            pizzaList: [],
            totalPrice: 0,
        }
        clientData.pizzaDay.orders.push(order);
    }
    const pizzaOrder: PizzaOrder = {
        varId: size.varId,
        name: pizza.name,
        size: size.size,
        price: size.price,
    }
    order.pizzaList.push(pizzaOrder);
    order.totalPrice += pizzaOrder.price;
    db.set(today, clientData);
    return clientData;
}

/**
 * Odstraní danou objednávku pizzy.
 *
 * @param login login uživatele
 * @param pizzaOrder objednávka pizzy
 */
export function removePizzaOrder(login: string, pizzaOrder: PizzaOrder) {
    const today = formatDate(getToday());
    const clientData: ClientData = db.get(today);
    if (!clientData.pizzaDay) {
        throw Error("Pizza day pro dnešní den neexistuje");
    }
    const orderIndex = clientData.pizzaDay.orders.findIndex(o => o.customer === login);
    if (orderIndex < 0) {
        throw Error("Nebyly nalezeny žádné objednávky pro uživatele " + login);
    }
    const order = clientData.pizzaDay.orders[orderIndex];
    const index = order.pizzaList.findIndex(o => o.name === pizzaOrder.name && o.size === pizzaOrder.size);
    if (index < 0) {
        throw Error("Objednávka s danými parametry nebyla nalezena");
    }
    const price = order.pizzaList[index].price;
    order.pizzaList.splice(index, 1);
    order.totalPrice -= price;
    if (order.pizzaList.length == 0) {
        clientData.pizzaDay.orders.splice(orderIndex, 1);
    }
    db.set(today, clientData);
    return clientData;
}

/**
 * Uzamkne možnost editovat objednávky pizzy.
 *
 * @param login login uživatele
 * @returns aktuální data pro uživatele
 */
export function lockPizzaDay(login: string) {
    const today = formatDate(getToday());
    const clientData: ClientData = db.get(today);
    if (!clientData.pizzaDay) {
        throw Error("Pizza day pro dnešní den neexistuje");
    }
    if (clientData.pizzaDay.creator !== login) {
        throw Error("Pizza day není spravován uživatelem " + login);
    }
    if (clientData.pizzaDay.state !== PizzaDayState.CREATED && clientData.pizzaDay.state !== PizzaDayState.ORDERED) {
        throw Error("Pizza day není ve stavu " + PizzaDayState.CREATED + " nebo " + PizzaDayState.ORDERED);
    }
    clientData.pizzaDay.state = PizzaDayState.LOCKED;
    db.set(today, clientData);
    return clientData;
}

/**
 * Odekmne možnost editovat objednávky pizzy.
 *
 * @param login login uživatele
 * @returns aktuální data pro uživatele
 */
export function unlockPizzaDay(login: string) {
    const today = formatDate(getToday());
    const clientData: ClientData = db.get(today);
    if (!clientData.pizzaDay) {
        throw Error("Pizza day pro dnešní den neexistuje");
    }
    if (clientData.pizzaDay.creator !== login) {
        throw Error("Pizza day není spravován uživatelem " + login);
    }
    if (clientData.pizzaDay.state !== PizzaDayState.LOCKED) {
        throw Error("Pizza day není ve stavu " + PizzaDayState.LOCKED);
    }
    clientData.pizzaDay.state = PizzaDayState.CREATED;
    db.set(today, clientData);
    return clientData;
}

/**
 * Nastaví stav pizza day na "pizzy objednány".
 *
 * @param login login uživatele
 * @returns aktuální data pro uživatele
 */
export function finishPizzaOrder(login: string) {
    const today = formatDate(getToday());
    const clientData: ClientData = db.get(today);
    if (!clientData.pizzaDay) {
        throw Error("Pizza day pro dnešní den neexistuje");
    }
    if (clientData.pizzaDay.creator !== login) {
        throw Error("Pizza day není spravován uživatelem " + login);
    }
    if (clientData.pizzaDay.state !== PizzaDayState.LOCKED) {
        throw Error("Pizza day není ve stavu " + PizzaDayState.LOCKED);
    }
    clientData.pizzaDay.state = PizzaDayState.ORDERED;
    db.set(today, clientData);
    callNotifikace({input:{udalost:UdalostEnum.OBJEDNANA_PIZZA,user:clientData?.pizzaDay?.creator}})
    return clientData;
}

/**
 * Nastaví stav pizza day na "pizzy doručeny".
 *
 * @param login login uživatele
 * @returns aktuální data pro uživatele
 */
export function finishPizzaDelivery(login: string) {
    const today = formatDate(getToday());
    const clientData: ClientData = db.get(today);
    if (!clientData.pizzaDay) {
        throw Error("Pizza day pro dnešní den neexistuje");
    }
    if (clientData.pizzaDay.creator !== login) {
        throw Error("Pizza day není spravován uživatelem " + login);
    }
    if (clientData.pizzaDay.state !== PizzaDayState.ORDERED) {
        throw Error("Pizza day není ve stavu " + PizzaDayState.ORDERED);
    }
    clientData.pizzaDay.state = PizzaDayState.DELIVERED;
    db.set(today, clientData);
    return clientData;
}

export function initIfNeeded() {
    const today = formatDate(getToday());
    if (!db.has(today)) {
        db.set(today, getEmptyData());
    }
}

export function removeChoice(login: string, data: ClientData) {
    for (let key of Object.keys(data.choices)) {
        if (data.choices[key] && data.choices[key].includes(login)) {
            const index = data.choices[key].indexOf(login);
            data.choices[key].splice(index, 1);
            if (data.choices[key].length == 0) {
                delete data.choices[key];
            }
        }
    }
    return data;
}

export function updateChoice(login: string, choice: Locations | null) {
    initIfNeeded();
    const today = formatDate(getToday());
    let data: ClientData = db.get(today);
    data = removeChoice(login, data);
    if (choice !== null) {
        if (!data.choices?.[choice]) {
            data.choices[choice] = [];
        }
        data.choices[choice].push(login);
    }
    db.set(today, data);
    return data;
}